"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveProjectPath = exports.resolveConfigurationFile = exports.resolveEnvironment = exports.resolveConfiguration = exports.toCamelCase = exports.capitalize = exports.toSnakeCase = exports.findArgumentValue = exports.combine = exports.traverseArgvMatching = exports.findLastIndex = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const util_1 = __importDefault(require("util"));
const debug_1 = __importDefault(require("./debug"));
const assertions_1 = require("./assertions");
/**
 * This is obviously a non-exhaustive list.
 */
const RECOGNIZED_CONFIGURATION_ATTRIBUTES = [
    "integrationFolder",
    "fixturesFolder",
    "supportFile",
    "testFiles",
    "ignoreTestFiles",
];
function findLastIndex(collection, predicate, beforeIndex = collection.length) {
    for (let i = beforeIndex - 1; i >= 0; --i) {
        if (predicate(collection[i])) {
            return i;
        }
    }
    return -1;
}
exports.findLastIndex = findLastIndex;
function* traverseArgvMatching(argv, name, allowEqual) {
    let beforeIndex = argv.length, matchingIndex;
    while ((matchingIndex = findLastIndex(argv, (arg) => arg.startsWith(name), beforeIndex)) !== -1) {
        if (argv[matchingIndex] === name) {
            if (argv.length - 1 === matchingIndex) {
                debug_1.default(`'${name}' argument missing`);
            }
            else {
                yield argv[matchingIndex + 1];
            }
        }
        else if (allowEqual && argv[matchingIndex][name.length] === "=") {
            yield argv[matchingIndex].slice(name.length + 1);
        }
        beforeIndex = matchingIndex;
    }
}
exports.traverseArgvMatching = traverseArgvMatching;
function* combine(...generators) {
    for (const generator of generators) {
        yield* generator;
    }
}
exports.combine = combine;
function findArgumentValue(argv, name, allowEqual) {
    for (const value of traverseArgvMatching(argv, name, allowEqual)) {
        return value;
    }
}
exports.findArgumentValue = findArgumentValue;
function toSnakeCase(value) {
    return value.replace(/[A-Z]/g, (letter) => `_${letter.toLowerCase()}`);
}
exports.toSnakeCase = toSnakeCase;
function capitalize(word) {
    return word.toLowerCase().replace(/\b\w/g, (l) => l.toUpperCase());
}
exports.capitalize = capitalize;
function toCamelCase(value) {
    return value
        .split("_")
        .map((word, index) => index === 0 ? word.toLocaleLowerCase() : capitalize(word))
        .join("");
}
exports.toCamelCase = toCamelCase;
function resolveConfiguration(options) {
    const { argv = process.argv, env = process.env, } = options;
    const projectPath = resolveProjectPath(options);
    const cliOrigin = Object.fromEntries(Array.from(combine(traverseArgvMatching(argv, "--config", true), traverseArgvMatching(argv, "-c", false)))
        .reverse()
        .flatMap((argument) => {
        const keypairExpr = /(?:^|,)([^=]+)=([^,$]+)/g;
        const entries = [];
        let match;
        while ((match = keypairExpr.exec(argument)) !== null) {
            if (RECOGNIZED_CONFIGURATION_ATTRIBUTES.includes(match[1])) {
                entries.push([match[1], match[2]]);
            }
        }
        return entries;
    }));
    const envPrefixExpr = /^cypress_(.+)/i;
    const envOrigin = Object.fromEntries(Object.entries(env)
        .filter((entry) => {
        return envPrefixExpr.test(entry[0]);
    })
        .map((entry) => {
        const match = entry[0].match(envPrefixExpr);
        assertions_1.assert(match, "cypress-cucumber-preprocessor: expected match after test");
        return [assertions_1.assertAndReturn(match[1]), entry[1]];
    })
        .map((entry) => {
        return [
            entry[0].includes("_") ? toCamelCase(entry[0]) : entry[0],
            entry[1],
        ];
    })
        .filter((entry) => {
        return RECOGNIZED_CONFIGURATION_ATTRIBUTES.includes(entry[0]);
    }));
    let configOrigin = {};
    const cypressConfigPath = path_1.default.join(projectPath, resolveConfigurationFile(options));
    if (fs_1.default.existsSync(cypressConfigPath)) {
        const content = fs_1.default.readFileSync(cypressConfigPath).toString("utf8");
        configOrigin = JSON.parse(content);
    }
    const configuration = Object.assign({
        integrationFolder: "cypress/integration",
        fixturesFolder: "cypress/fixtures",
        supportFile: "cypress/support/index.js",
        testFiles: "**/*.*",
        ignoreTestFiles: "*.hot-update.js",
    }, configOrigin, envOrigin, cliOrigin);
    debug_1.default(`resolved configuration of ${util_1.default.inspect(configuration)}`);
    return configuration;
}
exports.resolveConfiguration = resolveConfiguration;
function resolveEnvironment(options) {
    const { argv = process.argv, env = process.env, } = options;
    const projectPath = resolveProjectPath(options);
    const cliOrigin = Object.fromEntries(Array.from(combine(traverseArgvMatching(argv, "--env", true), traverseArgvMatching(argv, "-e", false)))
        .slice(0, 1)
        .flatMap((argument) => {
        const keypairExpr = /(?:^|,)([^=]+)=([^,$]+)/g;
        const entries = [];
        let match;
        while ((match = keypairExpr.exec(argument)) !== null) {
            entries.push([match[1], match[2]]);
        }
        return entries;
    }));
    const envPrefixExpr = /^cypress_(.+)/i;
    const envOrigin = Object.fromEntries(Object.entries(env)
        .filter((entry) => {
        return envPrefixExpr.test(entry[0]);
    })
        .map((entry) => {
        const match = entry[0].match(envPrefixExpr);
        assertions_1.assert(match, "cypress-cucumber-preprocessor: expected match after test");
        return [assertions_1.assertAndReturn(match[1]), entry[1]];
    }));
    const cypressConfigPath = path_1.default.join(projectPath, resolveConfigurationFile(options));
    let configOrigin = {};
    if (fs_1.default.existsSync(cypressConfigPath)) {
        const content = fs_1.default.readFileSync(cypressConfigPath).toString("utf8");
        const cypressConfig = JSON.parse(content);
        if (cypressConfig.env) {
            configOrigin = cypressConfig.env;
        }
    }
    const cypressEnvironmentFilePath = path_1.default.join(projectPath, "cypress.env.json");
    let cypressEnvOrigin = {};
    if (fs_1.default.existsSync(cypressEnvironmentFilePath)) {
        const content = fs_1.default
            .readFileSync(cypressEnvironmentFilePath)
            .toString("utf8");
        cypressEnvOrigin = JSON.parse(content);
    }
    const environment = Object.assign({}, cypressEnvOrigin, configOrigin, envOrigin, cliOrigin);
    debug_1.default(`resolved environment of ${util_1.default.inspect(environment)}`);
    return environment;
}
exports.resolveEnvironment = resolveEnvironment;
function resolveConfigurationFile(options) {
    const { argv = process.argv } = options;
    return (findArgumentValue(argv, "--config-file", true) ||
        findArgumentValue(argv, "-C", false) ||
        "cypress.json");
}
exports.resolveConfigurationFile = resolveConfigurationFile;
function resolveProjectPath(options) {
    const { argv = process.argv, cwd = process.cwd() } = options;
    const customProjectPath = findArgumentValue(argv, "--project", true) ||
        findArgumentValue(argv, "-P", false);
    return customProjectPath ? path_1.default.join(cwd, customProjectPath) : cwd;
}
exports.resolveProjectPath = resolveProjectPath;
