"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Registry = void 0;
const cucumber_expressions_1 = require("@cucumber/cucumber-expressions");
const tag_expressions_1 = __importDefault(require("@cucumber/tag-expressions"));
function parseHookArguments(optionsOrFn, maybeFn) {
    const noopNode = { evaluate: () => true };
    if (typeof optionsOrFn === "function") {
        if (maybeFn) {
            throw new Error("Unexpected argument for Before hook");
        }
        return { implementation: optionsOrFn, node: noopNode };
    }
    else if (typeof optionsOrFn === "object") {
        if (typeof maybeFn !== "function") {
            throw new Error("Unexpected argument for Before hook");
        }
        return {
            node: optionsOrFn.tags ? tag_expressions_1.default(optionsOrFn.tags) : noopNode,
            implementation: maybeFn,
        };
    }
    else {
        throw new Error("Unexpected argument for Before hook");
    }
}
class Registry {
    constructor() {
        this.methods = {
            defineStep: this.defineStep.bind(this),
            Step: this.runStepDefininition.bind(this),
            defineParameterType: this.defineParameterType.bind(this),
            Before: this.defineBefore.bind(this),
            After: this.defineAfter.bind(this),
        };
        this.parameterTypeRegistry = new cucumber_expressions_1.ParameterTypeRegistry();
        this.stepDefinitions = [];
        this.beforeHooks = [];
        this.afterHooks = [];
    }
    defineStep(description, implementation) {
        if (typeof description === "string") {
            this.stepDefinitions.push({
                expression: new cucumber_expressions_1.CucumberExpression(description, this.parameterTypeRegistry),
                implementation,
            });
        }
        else if (description instanceof RegExp) {
            this.stepDefinitions.push({
                expression: new cucumber_expressions_1.RegularExpression(description, this.parameterTypeRegistry),
                implementation,
            });
        }
        else {
            throw new Error("Unexpected argument for step definition");
        }
    }
    defineParameterType({ name, regexp, transformer, useForSnippets, preferForRegexpMatch, }) {
        if (typeof useForSnippets !== "boolean")
            useForSnippets = true;
        if (typeof preferForRegexpMatch !== "boolean")
            preferForRegexpMatch = false;
        this.parameterTypeRegistry.defineParameterType(new cucumber_expressions_1.ParameterType(name, regexp, null, transformer, useForSnippets, preferForRegexpMatch));
    }
    defineBefore(optionsOrFn, maybeFn) {
        this.beforeHooks.push(parseHookArguments(optionsOrFn, maybeFn));
    }
    defineAfter(optionsOrFn, maybeFn) {
        this.afterHooks.push(parseHookArguments(optionsOrFn, maybeFn));
    }
    resolveStepDefintion(text) {
        const matchingStepDefinitions = this.stepDefinitions.filter((stepDefinition) => stepDefinition.expression.match(text));
        if (matchingStepDefinitions.length === 0) {
            throw new Error(`Step implementation missing for: ${text}`);
        }
        else if (matchingStepDefinitions.length > 1) {
            throw new Error(`Multiple matching step definitions for: ${text}\n` +
                matchingStepDefinitions
                    .map((stepDefinition) => {
                    const { expression } = stepDefinition;
                    if (expression instanceof cucumber_expressions_1.RegularExpression) {
                        return ` ${expression.regexp}`;
                    }
                    else if (expression instanceof cucumber_expressions_1.CucumberExpression) {
                        return ` ${expression.source}`;
                    }
                })
                    .join("\n"));
        }
        else {
            return matchingStepDefinitions[0];
        }
    }
    runStepDefininition(world, text, argument) {
        const stepDefinition = this.resolveStepDefintion(text);
        const args = stepDefinition.expression
            .match(text)
            .map((match) => match.getValue(world));
        if (argument) {
            args.push(argument);
        }
        stepDefinition.implementation.apply(world, args);
    }
    resolveBeforeHooks(tags) {
        return this.beforeHooks.filter((beforeHook) => beforeHook.node.evaluate(tags));
    }
    resolveAfterHooks(tags) {
        return this.afterHooks.filter((beforeHook) => beforeHook.node.evaluate(tags));
    }
    runHook(world, hook) {
        hook.implementation.call(world);
    }
}
exports.Registry = Registry;
const globalPropertyName = "__cypress_cucumber_preprocessor_registry_dont_use_this";
const registry = globalThis[globalPropertyName] ||
    (globalThis[globalPropertyName] = new Registry());
exports.default = registry;
